==> lmtpd-0.9.19.tar.gz: lmtpd-0.9.19/re/re.c <==
/*
   Copyright 2000, 2001, 2002, 2003 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

static char const rcsid[] =
"$Id: re.c,v 1.14 2004/03/29 13:18:31 lwa Exp $";

#include "config.h"

#include <sys/types.h>
#include <regex.h>

#include <stddef.h>
#include <ctype.h>

#include "re.h"

extern RE re_extended;
extern RE re_basic;
#if WITH_PCRE
extern RE re_pcre;
extern RE re_pcre_utf8;
#endif
extern RE re_fnmatch;
extern RE re_rfc822;
extern RE re_true;
extern RE re_false;
extern RE re_error;
extern RE re_error_memory;
#if WITH_TRE
extern RE re_tre_extended;
char *re_tre_init(void);
char *re_tre_params(RE *re, char *s);
#endif

#if WITH_ONIGURUMA
extern RE *re_oni_choise(char **s);
#endif

extern RE re_strstr;

static void re_error_close(RE *re) {}
static int re_error_compile(RE *re, char *s, int i) {return re->err;}
static int re_error_exec(RE *re, char *s) {return -1;}

#if !WITH_DB
#undef WITH_REGEXP_DB_CACHE
#endif

#if WITH_REGEXP_DB_CACHE
extern int re_cache;
#endif

static RE re_error = {
  re_error_compile,
  re_error_exec, 
  re_error_close,
  0,
  RE_ERR_SYNTAX,
  NULL,
};

static RE re_error_memory = {
  re_error_compile,
  re_error_exec, 
  re_error_close,
  0,
  RE_ERR_MEMORY,
  "memory",
};

char *re_init(void) {
  char *ret = NULL;
#if WITH_TRE
  if ((ret = re_tre_init()) != NULL)
    return ret;
#endif
  return ret;
}


RE* re_new(char *pattern, char *type) {
  int icase = 1;
  RE *re;
  switch (*type) {
  case 'E':
    icase = 0;
  case 'e':
    re = &re_extended; break;
  case 'B':
    icase = 0;
  case 'b':
    re = &re_basic;  break;
#if WITH_PCRE
  case 'P':
    icase = 0;
  case 'p':
    if (type[1] == 'u') {
      re = &re_pcre_utf8;
    } else {
      re = &re_pcre;
    }
    break;
#endif
  case 'F':
    icase = 0; 
  case 'f':
    re = &re_fnmatch; break;
  case 'S':
    icase = 0; 
  case 's':
    re = &re_strstr; break;
  case 'a':
    icase = 0; 
  case 'A':
    re = &re_rfc822; break;
#if WITH_TRE
  case 'T':
    icase = 0;
  case 't':
    re = &re_tre_extended; 
    re_tre_params(re, type + 1);
    break;
#endif
  case '1':
    re = &re_true; break;
  case '0':
    re = &re_false; break;
  case '\0':
    re = &re_error_memory; break;
  default:
    re = &re_error;
  }
  re->compile(re, pattern, icase);
  return re;
}

#include <stdio.h>

RE *re_find(char **source) {
  char endchar='\0';
  char *src = *source;
  int icase = 1;
  char *re_start, *re_end=NULL;
  RE *re = &re_extended;

  if (*src == '/') {
    endchar = *src++;
  } else if (*src == '"' || *src == '\'') {
    endchar = *src++;
    re = &re_fnmatch;
  }


  re_start = src;
  while(*src) {
    if (*src == '\\') {
      src ++;
      if (*src =='n')
	*src = '\n';
      else if (*src == 't')
	*src = '\t';
      if (*src)
	src ++;
      continue;
    }
    if (endchar) {
      if (*src == endchar) {
	re_end = src++;
	break;
      }
    } else if (isspace((int)*src)) {
      re_end = src++;
      break;
    }
    src ++;
  }
  if (!re_end) {
    if (endchar) {
      *source = src;
      return &re_error;
    }
    re_end = src;
  } else if (endchar == '/') {
    if (*src && !isspace((int)*src)) {
      switch(*src) {
      case 'E': icase = 0;
      case 'e': re = &re_extended; src++; break;
      case 'B':	icase = 0;
      case 'b':	re = &re_basic; src ++; break;
#if WITH_PCRE
      case 'P':	icase = 0;
      case 'p':	if (src[1] == 'u') {
	  src++;
	  re = &re_pcre_utf8;
	} else {
	  re = &re_pcre;
	}
	src++;
	break;
#endif
      case 'F':	icase = 0; 
      case 'f':	re = &re_fnmatch; src++; break;
      case 'S':	icase = 0; 
      case 's':	re = &re_strstr; src++; break;
      case 'a':	icase = 0; 
      case 'A':	re = &re_rfc822; src ++; break;
#if WITH_ONIGURUMA
      case 'o':	icase = 0; 	
      case 'O':	src ++; re = re_oni_choise(&src); break;
#endif
#if WITH_TRE
      case 'T':	icase = 0;
      case 't': re = &re_tre_extended; src = re_tre_params(re, src+1); break;
#endif
      default:
	return &re_error;
      }
      if (*src && !isspace((int)*src)) {
     	return &re_error;
      }
    }
  }


#if WITH_REGEXP_DB_CACHE
  if (re_cache) {
    re = re_cache_search(re, *source, src, re_start, re_end, icase);
  }
  else 
#endif
    {
      char savedchar = *re_end;
      *re_end = 0;
      re->compile(re, re_start, icase);
      *re_end = savedchar;
    }

  if (re->err) {
    return re;
  }
  *source = src;
  return re;
}

==> lmtpd-0.10.2.tar.gz: lmtpd-0.10.2/re/re.c <==
/*
   Copyright 2000, 2001, 2002, 2003 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "config.h"

static char const rcsid[] UNUSED =
"$Id: re.c,v 1.15 2004/10/26 09:59:21 lwa Exp $";



#include <sys/types.h>
#include <regex.h>

#include <stddef.h>
#include <ctype.h>

#include "re.h"

extern RE re_extended;
extern RE re_basic;
#if WITH_PCRE
extern RE re_pcre;
extern RE re_pcre_utf8;
#endif
extern RE re_fnmatch;
extern RE re_rfc822;
extern RE re_true;
extern RE re_false;
extern RE re_error;
extern RE re_error_memory;
#if WITH_TRE
extern RE re_tre_extended;
char *re_tre_init(void);
char *re_tre_params(RE *re, char *s);
#endif

#if WITH_ONIGURUMA
extern RE *re_oni_choise(char **s);
#endif

extern RE re_strstr;

static void re_error_close(RE *re) {}
static int re_error_compile(RE *re, char *s, int i) {return re->err;}
static int re_error_exec(RE *re, char *s) {return -1;}

#if !WITH_DB
#undef WITH_REGEXP_DB_CACHE
#endif

#if WITH_REGEXP_DB_CACHE
extern int re_cache;
#endif

static RE re_error = {
  re_error_compile,
  re_error_exec, 
  re_error_close,
  0,
  RE_ERR_SYNTAX,
  NULL,
};

static RE re_error_memory = {
  re_error_compile,
  re_error_exec, 
  re_error_close,
  0,
  RE_ERR_MEMORY,
  "memory",
};

char *re_init(void) {
  char *ret = NULL;
#if WITH_TRE
  if ((ret = re_tre_init()) != NULL)
    return ret;
#endif
  return ret;
}


RE* re_new(char *pattern, char *type) {
  int icase = 1;
  RE *re;
  switch (*type) {
  case 'E':
    icase = 0;
  case 'e':
    re = &re_extended; break;
  case 'B':
    icase = 0;
  case 'b':
    re = &re_basic;  break;
#if WITH_PCRE
  case 'P':
    icase = 0;
  case 'p':
    if (type[1] == 'u') {
      re = &re_pcre_utf8;
    } else {
      re = &re_pcre;
    }
    break;
#endif
  case 'F':
    icase = 0; 
  case 'f':
    re = &re_fnmatch; break;
  case 'S':
    icase = 0; 
  case 's':
    re = &re_strstr; break;
  case 'a':
    icase = 0; 
  case 'A':
    re = &re_rfc822; break;
#if WITH_TRE
  case 'T':
    icase = 0;
  case 't':
    re = &re_tre_extended; 
    re_tre_params(re, type + 1);
    break;
#endif
  case '1':
    re = &re_true; break;
  case '0':
    re = &re_false; break;
  case '\0':
    re = &re_error_memory; break;
  default:
    re = &re_error;
  }
  re->compile(re, pattern, icase);
  return re;
}

#include <stdio.h>

RE *re_find(char **source) {
  char endchar='\0';
  char *src = *source;
  int icase = 1;
  char *re_start, *re_end=NULL;
  RE *re = &re_extended;

  if (*src == '/') {
    endchar = *src++;
  } else if (*src == '"' || *src == '\'') {
    endchar = *src++;
    re = &re_fnmatch;
  }


  re_start = src;
  while(*src) {
    if (*src == '\\') {
      src ++;
      if (*src =='n')
	*src = '\n';
      else if (*src == 't')
	*src = '\t';
      if (*src)
	src ++;
      continue;
    }
    if (endchar) {
      if (*src == endchar) {
	re_end = src++;
	break;
      }
    } else if (isspace((int)*src)) {
      re_end = src++;
      break;
    }
    src ++;
  }
  if (!re_end) {
    if (endchar) {
      *source = src;
      return &re_error;
    }
    re_end = src;
  } else if (endchar == '/') {
    if (*src && !isspace((int)*src)) {
      switch(*src) {
      case 'E': icase = 0;
      case 'e': re = &re_extended; src++; break;
      case 'B':	icase = 0;
      case 'b':	re = &re_basic; src ++; break;
#if WITH_PCRE
      case 'P':	icase = 0;
      case 'p':	if (src[1] == 'u') {
	  src++;
	  re = &re_pcre_utf8;
	} else {
	  re = &re_pcre;
	}
	src++;
	break;
#endif
      case 'F':	icase = 0; 
      case 'f':	re = &re_fnmatch; src++; break;
      case 'S':	icase = 0; 
      case 's':	re = &re_strstr; src++; break;
      case 'a':	icase = 0; 
      case 'A':	re = &re_rfc822; src ++; break;
#if WITH_ONIGURUMA
      case 'o':	icase = 0; 	
      case 'O':	src ++; re = re_oni_choise(&src); break;
#endif
#if WITH_TRE
      case 'T':	icase = 0;
      case 't': re = &re_tre_extended; src = re_tre_params(re, src+1); break;
#endif
      default:
	return &re_error;
      }
      if (*src && !isspace((int)*src)) {
     	return &re_error;
      }
    }
  }


#if WITH_REGEXP_DB_CACHE
  if (re_cache) {
    re = re_cache_search(re, *source, src, re_start, re_end, icase);
  }
  else 
#endif
    {
      char savedchar = *re_end;
      *re_end = 0;
      re->compile(re, re_start, icase);
      *re_end = savedchar;
    }

  if (re->err) {
    return re;
  }
  *source = src;
  return re;
}
